#!/bin/sh

# Common tests for emitting finished signal
unity-tool -n com.canonical.Unity.Lens.Applications -p /com/canonical/unity/lens/applications -c

unity-tool -n com.canonical.Unity.Lens.Applications -p /com/canonical/unity/lens/commands -c

